@@ -134,7 +134,8 @@ class Recipe:
                 os.makedirs(var_dir)
 
             instance_home = location
-            client_home = options.get('client-home', os.path.join(var_dir, self.name))
+            client_home = options.get('client-home', os.path.join(var_dir, 
+                                                                  self.name))
             if not os.path.exists(client_home):
                 os.makedirs(client_home)
 
@@ -172,7 +173,24 @@ class Recipe:
             ip_address = options.get('ip-address', '')
             if ip_address:
                 ip_address = 'ip-address %s' % ip_address
-
+            environment_vars = options.get('environment-vars', '')
+            if environment_vars:
+                # if the vars are all given on one line we need to do some work
+                if not '\n' in environment_vars:
+                    keys = []
+                    values = []
+                    env_vars = environment_vars.split()
+                    # split out the odd and even items into keys, values
+                    for var in env_vars:
+                        if divmod(env_vars.index(var) + 1, 2)[1]:
+                            keys.append(var)
+                        else:
+                            values.append(var)
+                    env_vars = zip(keys, values)
+                    environment_vars = '\n'.join(["%s %s" % (env_var[0], env_var[1])
+                                                 for env_var in env_vars])
+                environment_vars = environment_template % environment_vars
+            
             zope_conf_additional = options.get('zope-conf-additional', '')
 
             event_log_level = options.get('event-log-level', 'INFO')
@@ -185,12 +203,14 @@ class Recipe:
                 event_log_dir = os.path.dirname(event_file)
                 if not os.path.exists(event_log_dir):
                     os.makedirs(event_log_dir)
-                event_log = event_logfile % {'event_logfile': event_file, 'event_log_level': event_log_level}
+                event_log = event_logfile % {'event_logfile': event_file, 
+                                             'event_log_level': event_log_level}
             # custom log 
             else:
                 event_log = custom_event_log
- 
-            z_log_name = options.get('z2-log', os.path.sep.join(('log', self.name + '-Z2.log',)))
+            
+            z_log_name = os.path.sep.join(('log', self.name + '-Z2.log'))
+            z_log_name = options.get('z2-log', z_log_name)
             z_log = os.path.join(var_dir, z_log_name)
             z_log_dir = os.path.dirname(z_log)
             if not os.path.exists(z_log_dir):
@@ -207,9 +227,11 @@ class Recipe:
             else:
                 access_event_log = custom_access_event_log
 
-            default_zpublisher_encoding = options.get('default-zpublisher-encoding', 'utf-8')
+            default_zpublisher_encoding = options.get('default-zpublisher-encoding', 
+                                                      'utf-8')
             if default_zpublisher_encoding:
-                default_zpublisher_encoding = 'default-zpublisher-encoding %s' % default_zpublisher_encoding
+                default_zpublisher_encoding = 'default-zpublisher-encoding %s' %\
+                                              default_zpublisher_encoding
 
             relstorage = options.get('rel-storage')
             if relstorage:
@@ -250,7 +272,8 @@ class Recipe:
                     blob_storage = os.path.join(base_dir, blob_storage)
                     if not os.path.exists(blob_storage):
                         os.makedirs(blob_storage)
-                    storage_snippet = blob_storage_template % (blob_storage, file_storage)
+                    storage_snippet = blob_storage_template % (blob_storage, 
+                                                               file_storage)
                 
             zserver_threads = options.get('zserver-threads', '')
             if zserver_threads:
@@ -303,6 +326,9 @@ class Recipe:
                 # no zeo-client
                 zeo_client_client = ''
                 zeo_client_name = ''
+            
+            zodb_tmp_storage = options.get('zodb-temporary-storage', 
+                                           zodb_temporary_storage_template)
 
             template = zope_conf_template
 
@@ -342,17 +368,21 @@ class Recipe:
                                         zserver_threads = zserver_threads,
                                         zodb_cache_size = zodb_cache_size,
                                         zeo_client_name = zeo_client_name,
+                                        zodb_tmp_storage = zodb_tmp_storage,
                                         pid_file = pid_file,
                                         lock_file = lock_file,
+                                        environment_vars = environment_vars,
                                         zope_conf_additional = zope_conf_additional,)
 
         zope_conf_path = os.path.join(location, 'etc', 'zope.conf')
-        open(zope_conf_path, 'w').write(zope_conf)
+        try:
+            fd = open(zope_conf_path, 'w')
+            fd.write(zope_conf)
+        finally:
+            fd.close()
 
     def patch_binaries(self, ws_locations):
         location = self.options['location']
-        # XXX We need to patch the windows specific batch scripts
-        # and they need a different path seperator
         path =":".join(ws_locations)
         for script_name in ('runzope', 'zopectl'):
             script_path = os.path.join(location, 'bin', script_name)
@@ -372,17 +402,23 @@ class Recipe:
             f.write(script)
             f.close()
         # Patch Windows scripts
-        path =";".join(ws_locations)
         for script_name in ('runzope.bat', ):
             script_path = os.path.join(location, 'bin', script_name)
             script = open(script_path).read()
             # This could need some regex-fu
-            lines = [l for l in script.splitlines() if not l.startswith('@set PYTHON=')]
+            lines = [l for l in script.splitlines() 
+                     if not l.startswith('@set PYTHON=')]
             lines.insert(2, '@set PYTHON=%s' % self.options['executable'])
             script = '\n'.join(lines)
+
+            # Use servicewrapper.py instead of calling run.py directly
+            # so that sys.path gets properly set. We used to append
+            # all the eggs to PYTHONPATH in runzope.bat, but after
+            # everything was turned into eggs we exceeded the
+            # environment maximum size for cmd.exe.
             script = script.replace(
-                'PYTHONPATH=%SOFTWARE_HOME%',
-                'PYTHONPATH='+path+';%SOFTWARE_HOME%;%PYTHONPATH%'
+                "ZOPE_RUN = r'%s\\Zope2\\Startup\\run.py' % SOFTWARE_HOME",
+                "ZOPE_RUN = r'%s\\bin\\servicewrapper.py' % INSTANCE_HOME"
                 )
             f = open(script_path, 'w')
             f.write(script)
@@ -593,6 +629,17 @@ zeo_blob_storage_template="""
     </zeoclient>
 """.strip()
 
+zodb_temporary_storage_template="""
+<zodb_db temporary>
+    # Temporary storage database (for sessions)
+    <temporarystorage>
+      name temporary storage for sessioning
+    </temporarystorage>
+    mount-point /temp_folder
+    container-class Products.TemporaryFolder.TemporaryContainer
+</zodb_db>
+""".strip()
+
 event_logfile = """
   <logfile>
     path %(event_logfile)s
@@ -622,6 +669,12 @@ webdav_server_template = """
 </webdav-source-server>
 """
 
+environment_template = """
+<environment>
+    %s
+</environment>
+"""
+
 # The template used to build zope.conf
 zope_conf_template="""\
 instancehome %(instance_home)s
@@ -637,6 +690,7 @@ verbose-security %(verbose_security)s
 %(ip_address)s
 %(zserver_threads)s
 %(zeo_client_name)s
+%(environment_vars)s
 
 <eventlog>
   level %(event_log_level)s
@@ -666,14 +720,7 @@ verbose-security %(verbose_security)s
     mount-point /
 </zodb_db>
 
-<zodb_db temporary>
-    # Temporary storage database (for sessions)
-    <temporarystorage>
-      name temporary storage for sessioning
-    </temporarystorage>
-    mount-point /temp_folder
-    container-class Products.TemporaryFolder.TemporaryContainer
-</zodb_db>
+%(zodb_tmp_storage)s
 
 pid-filename %(pid_file)s
 lock-filename %(lock_file)s
